/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "authenticationjob.h"
#include "settings.h"
#include <kimap/loginjob.h>
#include <qsettings.h>
#include <kolab/errorhandler.h>

AuthenticationJob::AuthenticationJob(const QString &userName, KIMAP::Session *session, QObject* parent)
:   KJob(parent),
    mUserName(userName),
    mSession(session),
    mDoProxyAuth(true)
{

}

AuthenticationJob::AuthenticationJob(KIMAP::Session *session, QObject* parent)
:   KJob(parent),
    mSession(session),
    mDoProxyAuth(false)
{

}


void AuthenticationJob::start()
{
    Debug() << "logging in as " << mUserName << " using " << Settings::instance().getAuthorizationUser();
    KIMAP::LoginJob *loginJob = new KIMAP::LoginJob( mSession );
    if (mDoProxyAuth) {
        loginJob->setUserName( mUserName );
        loginJob->setAuthorizationName(Settings::instance().getAuthorizationUser());
    } else {
        loginJob->setUserName( Settings::instance().getAuthorizationUser() );
    }
    loginJob->setPassword(Settings::instance().getPassword());
    
    loginJob->setEncryptionMode( KIMAP::LoginJob::TlsV1 );
    loginJob->setAuthenticationMode( KIMAP::LoginJob::Plain );

    QObject::connect( loginJob, SIGNAL(result(KJob*)),
                      this, SLOT(onLoginDone(KJob*)) );
    loginJob->start();
}

void AuthenticationJob::onLoginDone(KJob* job)
{
    if ( job->error() ) {
        Warning() << "Failed to login: " << job->errorString();
        setError(KJob::UserDefinedError);
        emitResult();
        return;
    }
    Debug() << "authentication successful";
    emitResult();
}
