/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef FETCHMESSAGESJOB_H
#define FETCHMESSAGESJOB_H
#include <kjob.h>
#include <kmime/kmime_message.h>
#include <kimap/session.h>
#include <kimap/fetchjob.h>

class FetchMessagesJob: public KJob
{
    Q_OBJECT
public:
    explicit FetchMessagesJob(const QString &folder, KIMAP::Session *, QObject* parent = 0);
    virtual void start();
    QList<KMime::Message::Ptr> getMessages() const;
    KIMAP::MessageFlags getFlags(KMime::Message::Ptr) const;
    qint64 getImapUid(KMime::Message::Ptr) const;
    
private Q_SLOTS:
    void onSelectDone(KJob *job);
    void onHeadersReceived( const QString &mailBox,
                            const QMap<qint64, qint64> &uids,
                            const QMap<qint64, qint64> &sizes,
                            const QMap<qint64, KIMAP::MessageFlags> &flags,
                            const QMap<qint64, KIMAP::MessagePtr> &messages );
    void onHeadersFetchDone( KJob *job );
private:
    void checkJobDone();
    QString mFolder;
    KIMAP::Session *mSession;

    QMap<qint64, qint64> mUids;
    QMap<qint64, KIMAP::MessageFlags> mFlags;
    QMap<qint64, KIMAP::MessagePtr> mMessages;
};

#endif // FETCHMESSAGESJOB_H
