/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "coordinationjob.h"
#include "sourceserver.h"
#include "kolabserver.h"
#include "migrateuserjob.h"
#include <kolab/errorhandler.h>
#include <QDebug>

CoordinationJob::CoordinationJob(SourceServer *sourceServer, KolabServer *kolabServer, QObject* parent)
:   KJob(parent),
    mSourceServer(sourceServer),
    mKolabServer(kolabServer)
{

}

void CoordinationJob::start()
{
    const QStringList &userList = mSourceServer->getUserList();
    if (userList.isEmpty()) {
        Debug() << "no users found";
    } else {
        Debug() << "migrating the following users: ";
        qDebug() << userList;
    }
    foreach (const QString &user, userList) {
        MigrateUserJob *job = new MigrateUserJob(mSourceServer->getSourceAccount(user), mKolabServer->getAccount(user), this);
        Q_ASSERT(job);
        job->exec();
        if (job->error()) {
            Error() << job->errorString();
            continue;
        }
    }

    Debug() << "done";
    emitResult();
    emit done();
}

// #include "moc_coordinationjob.cxx"
// #include "coordinationjob.moc"
