/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "migrateuserjob.h"
#include "migratefolderjob.h"
#include "sourceaccount.h"
#include "kolabaccount.h"
#include <kolab/errorhandler.h>

MigrateUserJob::MigrateUserJob(SourceAccount *sourceAccount, KolabAccount *kolabAccount, QObject* parent)
:   KJob(parent),
    mSourceAccount(sourceAccount),
    mKolabAccount(kolabAccount)
{
    connect(mSourceAccount, SIGNAL(folderList(QStringList)), this, SLOT(migrateFolders(QStringList)));
}

void MigrateUserJob::start()
{
    if (!mKolabAccount || !mSourceAccount) {
        setError(KJob::UserDefinedError);
        setErrorText("Failed to get source or target account");
        emitResult();
        return;
    }
    mKolabAccount->cleanAccount();
    mKolabAccount->setupFolders();
    migrateFolders(mSourceAccount->lookupFolderList());
}

void MigrateUserJob::migrateFolders(const QStringList &folders)
{
    foreach (const QString &folder, folders) {
        MigrateFolderJob *job = new MigrateFolderJob(folder, mSourceAccount, mKolabAccount, this);
        job->exec();
    }
    mSourceAccount->logout();
    mKolabAccount->logout();
    emitResult();
}


// #include "migrateuserjob.moc"
