/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SOURCEACCOUNT_H
#define SOURCEACCOUNT_H

#include <QObject>
#include <QStringList>
#include <formathelpers.h>
#include <kolabobject.h>
#include <kimap/loginjob.h>
#include <kimap/listjob.h>
#include <kmime/kmime_message.h>
#include "object.h"

class SourceAccount: public QObject
{
    Q_OBJECT
public:
    explicit SourceAccount(QObject* parent = 0);

    virtual QStringList lookupFolderList();
    virtual QList<Object> getObjects(const QString &folder);
    virtual void logout();
    virtual QPair<Kolab::FolderType, QString> translateFolder(const QString& folder);
signals:
    void folderList(const QStringList &);
//     FindFoldersJob *getFolderList();
//     GetObjectsJob *getObjects(const QString &folder);
//     KolabObject convertObject();
//     QString getDestinationFolder(const QString &folder);
};

class TestAccount: public SourceAccount
{
    Q_OBJECT
public:
    explicit TestAccount(QObject* parent = 0);
    virtual QStringList lookupFolderList();
    virtual QList<Object> getObjects(const QString &folder);
    QStringList mFolderList;
};


class KolabSourceAccount: public SourceAccount
{
    Q_OBJECT
public:
    explicit KolabSourceAccount(KIMAP::Session *session, QObject* parent = 0);
    virtual QStringList lookupFolderList();
    virtual QList<Object> getObjects(const QString &folder);
    virtual QPair<Kolab::FolderType, QString> translateFolder(const QString& folder);

    void logout();
    
private slots:
    void mailBoxesReceived(const QList<KIMAP::MailBoxDescriptor> &descriptors, const QList< QList< QByteArray > > &flags);
protected:
    void init();
    Kolab::FolderType getFolderType(const QString &folder);
    KIMAP::Session *mSession;
    KIMAP::LoginJob::EncryptionMode mEncryptionMode;
    KIMAP::LoginJob::AuthenticationMode mAuthenticationMode;
    QList<KIMAP::MailBoxDescriptor> mMailboxes;
    QList<KIMAP::MailBoxDescriptor> mPersonalNamespaces;
    QMultiHash<QString, QString> mKolabFolders;
};

class ExchangeIMAPSourceAccount: public KolabSourceAccount
{
    Q_OBJECT
public:
    explicit ExchangeIMAPSourceAccount(KIMAP::Session *session, QObject* parent = 0);
    Kolab::FolderType getFolderType(const QString &folder);
    QPair<Kolab::ObjectType, KMime::Content*> getObjectType(const KMime::Message::Ptr &msg);
    virtual QList<Object> getObjects(const QString &folder);
    virtual QPair<Kolab::FolderType, QString> translateFolder(const QString& folder);
};

#endif // SOURCEACCOUNT_H
