/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef SOURCESERVER_H
#define SOURCESERVER_H

#include <kimap/session.h>
#include <kimap/loginjob.h>
#include <qsharedpointer.h>
#include <QStringList>
#include "sourceaccount.h"

class SourceServer: public QObject
{
    Q_OBJECT
public:
    explicit SourceServer(QObject* parent = 0);
//     void setAdminCredentials(const QString &username, const QString &pw);
    virtual SourceAccount *getSourceAccount(const QString &user);
    QStringList getUserList();
    virtual QStringList retrieveUserList();
    void setSingleUser(const QString &);
protected:
    QStringList mExplicitUsers;
};


class TestServer: public SourceServer
{
    Q_OBJECT
public:
    explicit TestServer(QObject* parent = 0);
    
    virtual SourceAccount *getSourceAccount(const QString &user);
    virtual QStringList retrieveUserList();
    QStringList mUsers;
};

class KolabSourceServer: public SourceServer
{
    Q_OBJECT
public:
    explicit KolabSourceServer(QObject* parent = 0);
    
    virtual SourceAccount *getSourceAccount(const QString &user);
    virtual QStringList retrieveUserList();

    void setHost(const QString &host, qint16 port);
    void setAdminCredentials(const QString &username, const QString &pw);
    void setEncryption(KIMAP::LoginJob::EncryptionMode);
    void setAuthentication(KIMAP::LoginJob::AuthenticationMode);
protected:
    void logout();
    KIMAP::Session *mSession;
    QString mHost;
    int mPort;
    QString mUsername;
    QString mPw;
    KIMAP::LoginJob::EncryptionMode mEncryptionMode;
    KIMAP::LoginJob::AuthenticationMode mAuthenticationMode;
};

class ExchangeIMAPSourceServer: public KolabSourceServer
{
    Q_OBJECT
public:
    explicit ExchangeIMAPSourceServer(QObject* parent = 0);
    
    virtual SourceAccount *getSourceAccount(const QString &user);
};

#endif // SOURCESERVER_H
